searchData={"items":[{"type":"module","title":"Livex.Handlers","doc":"Handlers for component events in Livex.\n\nThis module contains functions for handling events that are dispatched to components.","ref":"Livex.Handlers.html"},{"type":"function","title":"Livex.Handlers.handle_component_event/3","doc":"Handles component events by mapping parameters and calling pre_render.\n\nThis function is used internally by Livex to process events that are sent to components.","ref":"Livex.Handlers.html#handle_component_event/3"},{"type":"function","title":"Parameters - Livex.Handlers.handle_component_event/3","doc":"* `module` - The component module\n* `params` - The event parameters\n* `socket` - The LiveView socket","ref":"Livex.Handlers.html#handle_component_event/3-parameters"},{"type":"function","title":"Returns - Livex.Handlers.handle_component_event/3","doc":"* `{:noreply, socket}` - The updated socket after processing the event","ref":"Livex.Handlers.html#handle_component_event/3-returns"},{"type":"module","title":"Livex.JSX","doc":"Provides macros for client-side state updates and event emission in Livex components.\n\nThis module contains two main sets of functionality:\n\n1. **JSX.emit** - Emit events from components to parent views/components\n2. **JSX.assign_state** - Update component state directly from templates","ref":"Livex.JSX.html"},{"type":"module","title":"Event Emission (JSX.emit) - Livex.JSX","doc":"The `emit` macros allow components to emit events that can be handled by parent components\nor views. This creates a clean communication channel between child and parent components.","ref":"Livex.JSX.html#module-event-emission-jsx-emit"},{"type":"module","title":"Examples - Livex.JSX","doc":"```elixir\n# In a component template\n Cancel \n Save \n\n# In the parent component/view\n<.live_component \n  module={MyApp.ModalComponent} \n  id=\"my-modal\" \n  phx-close=\"handle_modal_close\"\n  phx-save=\"handle_modal_save\">\n  Modal content here\n</.live_component>\n\n# In the parent's handle_event\ndef handle_event(\"handle_modal_close\", _, socket) do\n  {:noreply, assign(socket, :modal_open, false)}\nend\n\ndef handle_event(\"handle_modal_save\", %{\"id\" => id, \"data\" => data}, socket) do\n  # Process the saved data\n  {:noreply, socket}\nend\n```","ref":"Livex.JSX.html#module-examples"},{"type":"module","title":"State Updates (JSX.assign_state) - Livex.JSX","doc":"The `assign_state` macros allow updating component state directly from templates,\nwithout needing to write handle_event callbacks for simple state changes.","ref":"Livex.JSX.html#module-state-updates-jsx-assign_state"},{"type":"module","title":"Examples - Livex.JSX","doc":"```elixir\n# Update a single value\n Expand \n\n# Update multiple values at once\n \n  Close\n \n\n# Conditional updates\n \n  {if @is_expanded, do: \"Close\", else: \"Expand\"}\n \n\n# Force a component refresh without changing state\n Refresh Component \n```","ref":"Livex.JSX.html#module-examples"},{"type":"module","title":"Usage - Livex.JSX","doc":"To use these macros, add `use Livex.JSX` to your module, or they are automatically\nimported when using `Livex.LivexView` or `Livex.LivexComponent`.","ref":"Livex.JSX.html#module-usage"},{"type":"macro","title":"Livex.JSX.assign_state/0","doc":"Creates a JS command to trigger a component update without changing any data.\n\nThis is useful for forcing a component to re-render.","ref":"Livex.JSX.html#assign_state/0"},{"type":"macro","title":"Examples - Livex.JSX.assign_state/0","doc":"```elixir\n Refresh Component \n```","ref":"Livex.JSX.html#assign_state/0-examples"},{"type":"macro","title":"Livex.JSX.assign_state/1","doc":"Creates a JS command to update component data from the client with a keyword list of options.\n\nThis macro allows updating multiple component state values directly from client-side events.","ref":"Livex.JSX.html#assign_state/1"},{"type":"macro","title":"Parameters - Livex.JSX.assign_state/1","doc":"* `opts` - A keyword list of key-value pairs to update in the component's state","ref":"Livex.JSX.html#assign_state/1-parameters"},{"type":"macro","title":"Examples - Livex.JSX.assign_state/1","doc":"```elixir\n \n  Expand Details\n \n```","ref":"Livex.JSX.html#assign_state/1-examples"},{"type":"macro","title":"Livex.JSX.assign_state/2","doc":"Creates a JS command to update a single component data value from the client.\n\nSee `assign_state/1` for updating multiple values at once.","ref":"Livex.JSX.html#assign_state/2"},{"type":"macro","title":"Parameters - Livex.JSX.assign_state/2","doc":"* `key` - The key to update in the component's state\n* `val` - The value to assign","ref":"Livex.JSX.html#assign_state/2-parameters"},{"type":"macro","title":"Examples - Livex.JSX.assign_state/2","doc":"```elixir\n Expand \n```","ref":"Livex.JSX.html#assign_state/2-examples"},{"type":"macro","title":"Livex.JSX.emit/1","doc":"","ref":"Livex.JSX.html#emit/1"},{"type":"macro","title":"Livex.JSX.emit/2","doc":"","ref":"Livex.JSX.html#emit/2"},{"type":"macro","title":"Livex.JSX.emit/3","doc":"Generates a JS command to emit a client-side event.","ref":"Livex.JSX.html#emit/3"},{"type":"macro","title":"Variants - Livex.JSX.emit/3","doc":"* `emit(js, event_suffix, opts)` - Chain on an existing JS struct with options\n* `emit(js, event_suffix)` - Chain on an existing JS struct without options\n* `emit(event_suffix, opts)` - Create a new JS chain with options\n* `emit(event_suffix)` - Create a new JS chain without options","ref":"Livex.JSX.html#emit/3-variants"},{"type":"macro","title":"Parameters - Livex.JSX.emit/3","doc":"* `js` - An existing Phoenix.LiveView.JS struct to chain operations on\n* `event_suffix` - The suffix of the event to emit (e.g., \"close\" for \"phx-close\")\n* `opts` - Options for the event, such as `values: %{...}`","ref":"Livex.JSX.html#emit/3-parameters"},{"type":"macro","title":"Examples - Livex.JSX.emit/3","doc":"```elixir\n# With existing JS chain and options\nJS.transition(...) |> JSX.emit(\"close\", values: %{reason: \"cancelled\"})\n\n# With existing JS chain, no options\nJS.transition(...) |> JSX.emit(\"close\")\n\n# New JS chain with options\nJSX.emit(\"close\", values: %{reason: \"cancelled\"})\n\n# New JS chain without options\nJSX.emit(\"close\")\n```","ref":"Livex.JSX.html#emit/3-examples"},{"type":"module","title":"Livex.LivexComponent","doc":"A module that enhances Phoenix LiveComponent with automatic state management and lifecycle improvements.","ref":"Livex.LivexComponent.html"},{"type":"module","title":"Features - Livex.LivexComponent","doc":"* **Declarative State Definition**: Define component state properties with type safety\n* **Component Properties (attr)**: Define expected properties passed from parent\n* **Simplified Lifecycle**: Consolidate data derivation logic in the `pre_render` callback\n* **Dependency-Aware Assignments**: Use `assign_new/4` to compute values only when dependencies change\n* **Simplified State Updates**: Use `JSX.assign_state` for direct state updates from templates\n* **Component Events**: Emit events to parent views with `push_emit` or `JSX.emit`\n* **PubSub Integration**: Subscribe to topics with dependency-aware `assign_topic`","ref":"Livex.LivexComponent.html#module-features"},{"type":"module","title":"State and Properties - Livex.LivexComponent","doc":"Livex components use two key concepts for managing data:\n\n1. **state** - Internal component state that persists across renders\n2. **attr** - Properties passed down from parent components/views","ref":"Livex.LivexComponent.html#module-state-and-properties"},{"type":"module","title":"Component State - Livex.LivexComponent","doc":"```elixir\n# Client-side state, survives reconnects but not refreshes\nstate :is_expanded, :boolean\nstate :selected_tab, :string\nstate :current_count, :integer\n```","ref":"Livex.LivexComponent.html#module-component-state"},{"type":"module","title":"Component Properties (attr) - Livex.LivexComponent","doc":"```elixir\n# Properties passed from parent\nattr :id, :string, required: true\nattr :item_id, :string\nattr :initial_value, :string\nattr :on_save, :any\n```","ref":"Livex.LivexComponent.html#module-component-properties-attr"},{"type":"module","title":"Lifecycle Management - Livex.LivexComponent","doc":"Livex simplifies the component lifecycle by consolidating data derivation and initial \nassignment logic into a `pre_render/1` callback. This is where you can initialize state \nbased on attr values or compute derived data.\n\n```elixir\ndef pre_render(socket) do\n  {:noreply,\n   socket\n   |> assign_new(:is_expanded, fn -> false end)\n   |> assign_new(:pending_selection, [:selected_value], &(&1.selected_value))\n   |> assign_new(:has_changes, fn -> false end)\n   |> then(fn socket ->\n     # Compute derived state\n     assign(socket, :has_changes, \n       socket.assigns.pending_selection != socket.assigns.selected_value)\n   end)}\nend\n```","ref":"Livex.LivexComponent.html#module-lifecycle-management"},{"type":"module","title":"Simplified State Updates - Livex.LivexComponent","doc":"Livex introduces `JSX.assign_state` for updating component state directly from templates:\n\n```elixir\n# Update a single value\n Expand \n\n# Update multiple values\n \n  Close\n \n\n# Conditional updates\n \n  {if @is_expanded, do: \"Close\", else: \"Expand\"}\n \n```","ref":"Livex.LivexComponent.html#module-simplified-state-updates"},{"type":"module","title":"Component Events - Livex.LivexComponent","doc":"Livex enhances event handling for components, allowing them to emit custom events \nthat parent LiveViews or LiveComponents can listen for:","ref":"Livex.LivexComponent.html#module-component-events"},{"type":"module","title":"Emitting from templates: - Livex.LivexComponent","doc":"```elixir\n \n  Save Changes\n \n```","ref":"Livex.LivexComponent.html#module-emitting-from-templates"},{"type":"module","title":"Emitting from Elixir code: - Livex.LivexComponent","doc":"```elixir\ndef handle_event(\"save_changes\", _, socket) do\n  # Process the data...\n  \n  # Emit an event to the parent\n  socket = push_emit(socket, :saved, %{id: socket.assigns.id})\n  {:noreply, assign(socket, :is_saving, false)}\nend\n```","ref":"Livex.LivexComponent.html#module-emitting-from-elixir-code"},{"type":"module","title":"Handling in parent: - Livex.LivexComponent","doc":"```elixir\n<.live_component\n  module={MyApp.FormComponent}\n  id=\"my-form\"\n  phx-saved=\"handle_form_saved\"\n/>\n\n# In the parent's handle_event\ndef handle_event(\"handle_form_saved\", %{\"id\" => id}, socket) do\n  # Handle the saved event\n  {:noreply, socket}\nend\n```","ref":"Livex.LivexComponent.html#module-handling-in-parent"},{"type":"module","title":"PubSub Integration - Livex.LivexComponent","doc":"Livex makes it easier for components to subscribe to PubSub topics with `assign_topic`:\n\n```elixir\ndef pre_render(socket) do\n  {:noreply,\n   socket\n   |> assign_new(:status_message, fn -> \"Connecting...\" end)\n   |> assign_topic(:doc_updates, [:document_id], fn assigns ->\n     \"document_updates:#{assigns.document_id}\"\n   end)}\nend\n\n# Handle PubSub messages\ndef handle_info({:doc_updates, %{message: msg}}, socket) do\n  {:noreply, assign(socket, :status_message, msg)}\nend\n```","ref":"Livex.LivexComponent.html#module-pubsub-integration"},{"type":"module","title":"Complete Example - Livex.LivexComponent","doc":"```elixir\ndefmodule MyApp.FormComponent do\n  use Livex.LivexComponent\n  \n  attr :id, :string, required: true\n  attr :item_id, :string\n  attr :on_save, :any\n  \n  state :is_expanded, :boolean\n  state :form_data, :map\n  state :is_valid, :boolean\n  \n  def pre_render(socket) do\n    {:noreply,\n     socket\n     |> assign_new(:is_expanded, fn -> false end)\n     |> assign_new(:form_data, [:item_id], fn assigns ->\n       if assigns.item_id, do: load_item(assigns.item_id), else: %{}\n     end)\n     |> assign_new(:is_valid, [:form_data], fn assigns ->\n       validate_form(assigns.form_data)\n     end)}\n  end\n  \n  def render(assigns) do\n    ~H\"\"\"\n     \n       Edit Item \n      \n       \n        <!-- Form fields here -->\n       \n      \n       \n         \n          Save\n         \n        \n         \n          Cancel\n         \n       \n     \n    \"\"\"\n  end\n  \n  def handle_event(\"save\", _, socket) do\n    # Process the save...\n    if socket.assigns.on_save do\n      socket.assigns.on_save.(socket.assigns.form_data)\n    end\n    \n    # Emit an event to the parent\n    socket = push_emit(socket, :saved, %{id: socket.assigns.id})\n    {:noreply, socket}\n  end\n  \n  defp load_item(id) do\n    # Load item data\n  end\n  \n  defp validate_form(data) do\n    # Validate form data\n  end\nend\n```","ref":"Livex.LivexComponent.html#module-complete-example"},{"type":"function","title":"Livex.LivexComponent.inject_after_div/3","doc":"","ref":"Livex.LivexComponent.html#inject_after_div/3"},{"type":"function","title":"Livex.LivexComponent.override_update/4","doc":"","ref":"Livex.LivexComponent.html#override_update/4"},{"type":"function","title":"Livex.LivexComponent.override_update/5","doc":"","ref":"Livex.LivexComponent.html#override_update/5"},{"type":"module","title":"Livex.LivexView","doc":"A module that enhances Phoenix LiveView with automatic state management and lifecycle improvements.","ref":"Livex.LivexView.html"},{"type":"module","title":"Features - Livex.LivexView","doc":"* **Declarative State Definition**: Define state properties with type safety and URL persistence options\n* **Automatic URL State Management**: State marked with `url?: true` is automatically persisted to the URL\n* **Simplified Lifecycle**: Consolidate data derivation logic in the `pre_render` callback\n* **Dependency-Aware Assignments**: Use `assign_new/4` to compute values only when dependencies change\n* **Reactive Data Streams**: Use `stream_new/4` to automatically update streams when dependencies change\n* **Component State Management**: Define component modules as state types for parent-controlled components","ref":"Livex.LivexView.html#module-features"},{"type":"module","title":"State Management - Livex.LivexView","doc":"Livex allows you to declaratively define state properties for your LiveView:\n\n```elixir\n# State stored in URL, survives refreshes, typed as :integer\nstate :page_number, :integer, url?: true\n\n# State stored in URL, survives refreshes, typed as :string\nstate :category_filter, :string, url?: true\n\n# Client-side state, survives reconnects but not refreshes, typed as :boolean\nstate :show_advanced_search, :boolean\n\n# Component state type - parent can control this component's attributes\nstate :edit_form, MyApp.Components.EditForm\n```\n\n* `url?: true` - State is stored in the URL query string, making it bookmarkable and persistent across page refreshes\n* `url?: false` (or omitted) - State is stored only in client-side LiveView state, surviving reconnects but not refreshes\n* **Type System** - Livex handles casting state from URL (string) or client-side representation to its defined Elixir type\n* **Component State** - When a component module is used as a type, the parent can control that component's attributes","ref":"Livex.LivexView.html#module-state-management"},{"type":"module","title":"Lifecycle Management - Livex.LivexView","doc":"Livex simplifies the LiveView lifecycle by consolidating data derivation and initial assignment logic \ninto a `pre_render/1` callback. This largely replaces both `mount/3` and `handle_params/3`, providing \na single place to handle state initialization and data derivation.\n\nThe typical flow becomes: render -> event -> reducer (handle_event, handle_info, handle_async) -> pre_render -> render.\n\n```elixir\ndef pre_render(socket) do\n  {:noreply,\n   socket\n   |> assign_new(:page_title, fn -> \"Product Catalog\" end)\n   |> assign_new(:selected_category, fn -> \"all\" end)\n   |> assign_new(:products, [:selected_category], fn assigns ->\n     # This function only runs if selected_category changes\n     Products.list_by_category(assigns.selected_category)\n   end)}\nend\n```","ref":"Livex.LivexView.html#module-lifecycle-management"},{"type":"module","title":"Dependency-Aware Assignments - Livex.LivexView","doc":"The `assign_new/4` function extends Phoenix.Component.assign_new/3 with dependency tracking:\n\n```elixir\n# Runs only when user_id changes\nassign_new(socket, :profile, [:user_id], fn assigns ->\n  Accounts.get_user_profile(assigns.user_id)\nend)\n\n# Initial assignment (no dependencies)\nassign_new(socket, :is_admin_view, fn -> false end)\n```","ref":"Livex.LivexView.html#module-dependency-aware-assignments"},{"type":"module","title":"Reactive Data Streams - Livex.LivexView","doc":"The `stream_new/4` function enhances LiveView's stream/4 with dependency tracking:\n\n```elixir\n# Stream resets and repopulates when filter_category or filter_price_range changes\nstream_new(socket, :products, [:filter_category, :filter_price_range], fn assigns ->\n  filter_products(\n    Products.list_all(),\n    assigns.filter_category,\n    assigns.filter_price_range\n  )\nend)\n```","ref":"Livex.LivexView.html#module-reactive-data-streams"},{"type":"module","title":"Usage Example - Livex.LivexView","doc":"```elixir\ndefmodule MyApp.ProductListingView do\n  use Livex.LivexView\n  \n  state :selected_category, :string, url?: true\n  state :sort_by, :atom, url?: true\n  state :modal_open, :boolean\n  state :edit_form, MyApp.Components.EditForm\n  \n  def pre_render(socket) do\n    {:noreply,\n     socket\n     |> assign_new(:modal_open, fn -> false end)\n     |> assign_new(:page_title, fn -> \"Product Catalog\" end)\n     |> assign_new(:selected_category, fn -> \"all\" end)\n     |> assign_new(:sort_by, fn -> :name_asc end)\n     |> stream_new(:products, [:selected_category, :sort_by], fn assigns ->\n       Products.list_available_products(\n         category: assigns.selected_category,\n         order_by: assigns.sort_by\n       )\n     end)}\n  end\n  \n  def render(assigns) do\n    ~H\"\"\"\n     \n       {@page_title} \n      \n      <.live_component\n        :if={@edit_form}\n        module={MyApp.Components.EditForm}\n        id=\"edit-form\"\n        {@edit_form}\n        phx-close=\"close_form\"\n      />\n      \n      <!-- Product listing and filters -->\n     \n    \"\"\"\n  end\n  \n  def handle_event(\"close_form\", _, socket) do\n    {:noreply, assign(socket, :edit_form, nil)}\n  end\nend\n```","ref":"Livex.LivexView.html#module-usage-example"},{"type":"function","title":"Livex.LivexView.assign_from_uri/4","doc":"","ref":"Livex.LivexView.html#assign_from_uri/4"},{"type":"function","title":"Livex.LivexView.on_mount/4","doc":"","ref":"Livex.LivexView.html#on_mount/4"},{"type":"module","title":"Livex.ParamsMapper","doc":"Handles parameter mapping and type casting for Livex components and views.\n\nThis module is responsible for converting parameters between different formats\nand ensuring proper type casting based on component attribute definitions.","ref":"Livex.ParamsMapper.html"},{"type":"function","title":"Livex.ParamsMapper.cast/2","doc":"Cast a value to the specified type.","ref":"Livex.ParamsMapper.html#cast/2"},{"type":"function","title":"Livex.ParamsMapper.decode_json/2","doc":"Decode a JSON string and verify its type.","ref":"Livex.ParamsMapper.html#decode_json/2"},{"type":"function","title":"Livex.ParamsMapper.map_params/3","doc":"Map and cast parameters for a given component, optionally scoped by an `id`.","ref":"Livex.ParamsMapper.html#map_params/3"},{"type":"function","title":"Livex.ParamsMapper.parse_bool/1","doc":"Parse a string into a boolean value.","ref":"Livex.ParamsMapper.html#parse_bool/1"},{"type":"function","title":"Livex.ParamsMapper.parse_iso/2","doc":"Parse an ISO formatted string using the provided function.","ref":"Livex.ParamsMapper.html#parse_iso/2"},{"type":"function","title":"Livex.ParamsMapper.safe_atom/1","doc":"Safely convert a string to an existing atom.","ref":"Livex.ParamsMapper.html#safe_atom/1"},{"type":"function","title":"Livex.ParamsMapper.safe_parse/2","doc":"Safely parse a value using the provided function.","ref":"Livex.ParamsMapper.html#safe_parse/2"},{"type":"module","title":"Livex.RenderedManipulator","doc":"A utility library for manipulating Phoenix.LiveView.Rendered structs.","ref":"Livex.RenderedManipulator.html"},{"type":"function","title":"Livex.RenderedManipulator.attribute_snippets/2","doc":"","ref":"Livex.RenderedManipulator.html#attribute_snippets/2"},{"type":"function","title":"Livex.RenderedManipulator.format_attribute/4","doc":"Formats an attribute for inclusion in an HTML tag.","ref":"Livex.RenderedManipulator.html#format_attribute/4"},{"type":"function","title":"Parameters - Livex.RenderedManipulator.format_attribute/4","doc":"* `name` - The attribute name\n* `value` - The attribute value\n* `prefix` - Optional prefix for the attribute name\n* `type` - Optional type for the attribute name","ref":"Livex.RenderedManipulator.html#format_attribute/4-parameters"},{"type":"function","title":"Returns - Livex.RenderedManipulator.format_attribute/4","doc":"* A formatted attribute string","ref":"Livex.RenderedManipulator.html#format_attribute/4-returns"},{"type":"function","title":"Livex.RenderedManipulator.manipulate_rendered/5","doc":"Manipulates a Phoenix.LiveView.Rendered struct by either injecting attributes or wrapping content.","ref":"Livex.RenderedManipulator.html#manipulate_rendered/5"},{"type":"function","title":"Options - Livex.RenderedManipulator.manipulate_rendered/5","doc":"* `:inject` - Injects attributes into the first static chunk\n  * `:wrap` - Wraps the content with a new tag that has the specified attributes","ref":"Livex.RenderedManipulator.html#manipulate_rendered/5-options"},{"type":"function","title":"Parameters - Livex.RenderedManipulator.manipulate_rendered/5","doc":"* `mode` - Either `:inject` or `:wrap`\n  * `rendered` - The Phoenix.LiveView.Rendered struct to manipulate\n  * `attributes` - List of attribute strings to add\n  * `options` - Additional options depending on the mode:\n    * For `:inject` mode:\n      * `hook_name` - Optional hook name to add\n    * For `:wrap` mode:\n      * `tag` - The HTML tag to wrap content with\n      * `hook_name` - Optional hook name to add\n      * `id` - Optional ID to add to the wrapping tag","ref":"Livex.RenderedManipulator.html#manipulate_rendered/5-parameters"},{"type":"function","title":"Livex.RenderedManipulator.maybe_add_dom_id/1","doc":"Conditionally adds an ID attribute to an HTML element.\n\nReturns an ID attribute string if the ID is provided, otherwise returns an empty string.","ref":"Livex.RenderedManipulator.html#maybe_add_dom_id/1"},{"type":"function","title":"Examples - Livex.RenderedManipulator.maybe_add_dom_id/1","doc":"iex> maybe_add_dom_id(\"my-element\")\n    \" id=\\\"my-element\\\"\"\n    \n    iex> maybe_add_dom_id(nil)\n    \"\"","ref":"Livex.RenderedManipulator.html#maybe_add_dom_id/1-examples"},{"type":"function","title":"Livex.RenderedManipulator.maybe_prepend_route/2","doc":"","ref":"Livex.RenderedManipulator.html#maybe_prepend_route/2"},{"type":"function","title":"Livex.RenderedManipulator.wrap_in_div/3","doc":"","ref":"Livex.RenderedManipulator.html#wrap_in_div/3"},{"type":"module","title":"Livex.Utils","doc":"Utility functions for Livex components and views.\n\nThis module provides helper functions for event handling, JavaScript execution,\nand state management with dependency tracking.","ref":"Livex.Utils.html"},{"type":"function","title":"Livex.Utils.assign_new/4","doc":"Assigns a new value to the socket if it doesn't exist or if any of its dependencies have changed.\n\nThis function is similar to Phoenix.Component.assign_new/3, but with dependency tracking.\nIt will recompute the value if any of the dependencies have changed in the socket's assigns.","ref":"Livex.Utils.html#assign_new/4"},{"type":"function","title":"Parameters - Livex.Utils.assign_new/4","doc":"* `socket` - The LiveView socket\n* `key` - The key to assign the value to\n* `deps` - A list of keys that this value depends on\n* `fun` - A function that returns the value to assign","ref":"Livex.Utils.html#assign_new/4-parameters"},{"type":"function","title":"Examples - Livex.Utils.assign_new/4","doc":"```elixir\ndef pre_render(socket) do\n  {:noreply,\n   assign_new(socket, :location, [:location_id], fn assigns -> \n     MyApp.Domain.get_location!(assigns.location_id)\n   end)}\nend\n```","ref":"Livex.Utils.html#assign_new/4-examples"},{"type":"function","title":"Livex.Utils.push_emit/3","doc":"Emits a client-side event from the server.\n\nThis function allows components to emit events that can be handled by parent components\nor views, similar to how JSX.emit works but from server-side code.","ref":"Livex.Utils.html#push_emit/3"},{"type":"function","title":"Parameters - Livex.Utils.push_emit/3","doc":"* `socket` - The LiveView socket\n* `event` - The event suffix (will be looked up in assigns as \"phx-#{event}\")\n* `opts` - Options to pass to the event (default: [])","ref":"Livex.Utils.html#push_emit/3-parameters"},{"type":"function","title":"Examples - Livex.Utils.push_emit/3","doc":"```elixir\ndef handle_event(\"close\", _, socket) do\n  {:noreply, socket |> push_emit(:close)}\nend\n```","ref":"Livex.Utils.html#push_emit/3-examples"},{"type":"function","title":"Livex.Utils.push_js/2","doc":"Pushes a JavaScript command to be executed on the client.","ref":"Livex.Utils.html#push_js/2"},{"type":"function","title":"Parameters - Livex.Utils.push_js/2","doc":"* `socket` - The LiveView socket\n* `js` - The Phoenix.LiveView.JS struct containing operations to execute","ref":"Livex.Utils.html#push_js/2-parameters"},{"type":"function","title":"Returns - Livex.Utils.push_js/2","doc":"* The updated socket","ref":"Livex.Utils.html#push_js/2-returns"},{"type":"function","title":"Livex.Utils.stream_new/4","doc":"","ref":"Livex.Utils.html#stream_new/4"},{"type":"extras","title":"Livex","doc":"# Livex\n\n**This is not production code. It's a (fully functional) DX experiment to be able\nto experiment with the combination of various features. But uses elixir trickery\nand rather too much knowledge of liveview internals to make it work. Ideally the\nend state is either for the features to be added to liveview, or for the\ncapability to add the features to be added.**\n\nLivex is a library that provides LiveView components and views with\nenhanced state management, streamlining the persistence of state to the client\nallowing automatic recovery of both url based and non url based client state.\nIt allows for more a more declarative style based on the render -> reducer ->\nrender cycle, being robust in the face of reconnections without needing to hand\npersist state to the client via push_patch().","ref":"readme.html"},{"type":"extras","title":"Why Livex? - Livex","doc":"LiveView is a powerful platform for building interactive web applications.\nHowever, as applications grow, certain patterns can become cumbersome:\n\n- **URL State Management:** While storing state in the URL is crucial for\n  resilience (surviving refreshes, back button), manipulating individual query\n  parameters can be less ergonomic than direct state updates via assign().\n- **LiveComponent State:** Managing state within `LiveComponent`s robustly often\n  involves delegating to the parent and manually merging state concerns into the\n  URL query string, which can become complex.\n- **Clear State Patterns:** Defining clear patterns for different types of state\n  (parent-owned, child-owned, persistent, ephemeral) and how they transition can\n  be challenging.\n- **Derived State Updates:** Reacting to changes in URL parameters or component\n  arguments to refresh externally stored or derived state is often handled\n  imperatively, whereas a more declarative, event-driven approach could be\n  beneficial.\n- **Component PubSub:** Enabling components to be self-sufficient in terms of\n  real-time messaging via PubSub can require boilerplate.\n\nLivex aims to address these points by introducing features that promote a more\ndeclarative and streamlined developer experience, fitting within the existing\nLiveView paradigm while borrowing well-established concepts. It's intended\nto be a proof of concept of features that would be natural evolutions of\nLiveView and hopefully may lead to such features being included.","ref":"readme.html#why-livex"},{"type":"extras","title":"Installation - Livex","doc":"The package can be installed by adding `livex` to your list of dependencies in\n`mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:livex, \"~> 0.1.2\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Demo - Livex","doc":"To see Livex in action with concrete examples, check out the demo application at\nhttps://github.com/u2i/livex_demo.","ref":"readme.html#demo"},{"type":"extras","title":"Features - Livex","doc":"","ref":"readme.html#features"},{"type":"extras","title":"Declarative State Definition (state) - Livex","doc":"Livex allows you to declaratively define state properties for your LivexView or\nLivexComponent.\n\n```elixir\ndefmodule MyAppWeb.ArticleView do\n  use MyAppWeb, :livex_view # Or use Livex.LivexView directly\n\n  # State stored in URL, survives refreshes, typed as :integer\n  state :page_number, :integer, url?: true\n  # State stored in URL, survives refreshes, typed as :string\n  state :category_filter, :string, url?: true\n  # Client-side state, survives reconnects but not refreshes, typed as :boolean\n  state :show_advanced_search, :boolean\n\n  # Example of a component state type.\n  # The `MyAppWeb.Components.CommentForm` component's attrs will be managed here.\n  # This allows the parent view to control the initial state or reset the form.\n  state :comment_form, MyAppWeb.Components.CommentForm\nend\n\ndefmodule MyAppWeb.Components.InteractiveCard do\n  use MyAppWeb, :livex_component # Or use Livex.LivexComponent directly\n\n  # State specific to this card instance, can be stored in URL if needed\n  state :display_mode, :atom # :summary or :detailed\n  state :user_rating, :integer\n\n  # rest of your component code...\nend\n```\n\n- `state  ,  `: Defines a piece of state.\n- `url?: true`: Indicates the state should be stored in the URL query string.\n  This makes the state bookmarkable and persistent across page refreshes.\n- `url?: false` (or omitted): Indicates the state is stored only on the\n  client-side LiveView state. It will survive client-server reconnects but not\n  full page refreshes.\n- **Type System**: Livex handles casting the state from the URL (string) or\n  client-side representation back to its defined Elixir type.\n- **Component State**: You can define a state property as a component module\n  (e.g., `state :new_comment_form, MyAppWeb.Components.CommentForm`). Livex will\n  then manage the attributes (attr) of this component as part of the parent's\n  state. When this state is updated (e.g., via JSX.assign_state or a direct\n  assign in an event), Livex facilitates the re-rendering of the child component\n  with the new attribute values.\n- **Initial Values**: Default values are not directly supported in the state\n  macro (yet). To set an initial value, use assign_new/2 within the pre_render/1\n  callback without dependencies, e.g., `assign_new(socket, :my_state, fn -> \ninitial_value end)`.","ref":"readme.html#declarative-state-definition-state"},{"type":"extras","title":"Declarative Data Derivation and Lifecycle Management (pre_render, assign_new, stream_new) - Livex","doc":"Livex aims to simplify the LiveView/LiveComponent lifecycle (mount, update,\nhandle_params) by consolidating data derivation and initial assignment logic\ninto a pre_render/1 callback, coupled with enhanced assign and stream functions.\n\nThe typical flow becomes: render -> event -> reducer (handle_event, handle_info,\nhandle_async) -> pre_render -> render.\n\nThe pre_render callback largely replaces both mount and handle_params, providing\na single place to handle state initialization and data derivation. The standard\nLiveView callbacks still exist and can be used when needed, but pre_render\nprovides a more declarative approach.\n\nExample:\n\n```elixir\ndefmodule MyAppWeb.ProductListingView do\n  use MyAppWeb, :livex_view\n\n  state :selected_category, :string, url?: true\n  state :sort_by, :atom, url?: true\n  state :modal_open, :boolean\n\n  def pre_render(socket) do\n    {:noreply,\n     socket\n     |> assign_new(:modal_open, fn -> false end)\n     |> assign_new(:page_title, fn -> \"Product Catalog\" end)\n     |> assign_new(:selected_category, fn -> \"all\" end)\n     |> assign_new(:sort_by, fn -> :name_asc end)\n     |> stream_new(:products, [:selected_category, :sort_by], fn assigns ->\n       # This function re-runs if selected_category or sort_by changes\n       Products.list_available_products(\n         category: assigns.selected_category,\n         order_by: assigns.sort_by\n       )\n     end)}\n  end\n\n  # ... rest of the view code\nend\n```\n\n- **assign_new/4**: Works like Phoenix.Component.assign_new/3,\n  but with an explicit list of dependencies (assign keys). The assignment\n  function will only re-run if one of these dependencies has changed. This\n  memoization helps avoid redundant computations or data fetching. Can also be\n  used without dependencies to set initial values.\n\nExample:\n\n```elixir\ndefmodule MyAppWeb.UserProfileView do\n  use MyAppWeb, :livex_view\n\n  state :user_id, :string, url?: true # e.g., from URL like /users/:user_id\n\n  def pre_render(socket) do\n    {:noreply,\n     socket\n     |> assign_new(:profile, [:user_id], fn assigns ->\n       # This function only runs if assigns.user_id changes\n       Accounts.get_user_profile(assigns.user_id)\n     end)\n     |> assign_new(:activity_feed, [:user_id], fn assigns ->\n       Activity.fetch_feed_for_user(assigns.user_id)\n     end)\n     |> assign_new(:is_admin_view, fn -> false end) # Initial assignment\n    }\n  end\n  # ... render, handle_event, etc.\nend\n```\n\nIf :user_id (which is a state field) changes, the functions to fetch :profile\nand :activity_feed will be re-executed.\n\n- **stream_new/4**: An enhancement to LiveView's stream/4. It takes an\n  additional list of dependencies (assign keys from state or attr). The stream\n  will be reset and re-populated using the provided function if any of these\n  dependencies change. This is particularly useful for streams whose contents\n  depend on filter parameters or other dynamic data.\n\nExample:\n\n```elixir\ndef pre_render(socket) do\n  {:noreply,\n   socket\n   # Set initial values for filters if not present in URL\n   |> assign_new(:filter_category, fn -> nil end)\n   |> assign_new(:filter_price_range, fn -> nil end)\n   # Stream depends on the potentially initialized state values\n   |> stream_new(:products, [:filter_category, :filter_price_range], fn assigns ->\n     filter_products(\n       Products.list_all(),\n       assigns.filter_category,\n       assigns.filter_price_range\n     )\n   end)}\nend\n```\n\nIf either :filter_category or :filter_price_range changes, the :products stream will be\nautomatically updated by re-invoking the stream configuration function with the\nnew assigns.","ref":"readme.html#declarative-data-derivation-and-lifecycle-management-pre_render-assign_new-stream_new"},{"type":"extras","title":"Component Properties (attr) - Livex","doc":"Used in LivexComponent to define properties passed down from a parent. These are\nanalogous to \"props\" in React and are fully controlled by the parent.\n\n```elixir\n# In a filter component\nattr :id, :string\nattr :selected_value, :string\nattr :options, :list\nattr :title, :string\n```\n\nComponents can use pre_render to initialize state based on attr values:\n\n```elixir\ndef pre_render(socket) do\n  {:noreply,\n   socket\n   |> assign_new(:is_open, fn -> false end)\n   |> assign_new(:pending_selection, [:selected_value], &(&1.selected_value))\n   |> assign_new(:has_changes, fn -> false end)\n   |> then(fn socket ->\n     # Compute derived state\n     assign(socket, :has_changes, socket.assigns.pending_selection != socket.assigns.selected_value)\n   end)}\nend\n```\n\n#### Component Modules as State Types\n\nOne of the powerful features of Livex is the ability to define a state property\nusing a component module as its type. This creates a tight integration between\nparent views and their child components.\n\nWhen you define a state property with a component module as its type:\n\n```elixir\n# In parent view\nstate :edit_form, MyAppWeb.Components.EditForm\n```\n\nLivex will:\n\n1. Automatically manage the component's attributes as part of the parent's state\n2. Allow the parent to control the component's initial state or reset it\n3. Ensure proper re-rendering when the component state changes\n\nThe component itself defines its expected attributes using `attr`:\n\n```elixir\n# In MyAppWeb.Components.EditForm\nattr :id, :string, required: true\nattr :item_id, :string\nattr :action, :atom, default: :new\nattr :on_save, :any\n```\n\nWhen the parent renders this component, it can pass the state directly:\n\n```elixir\n<.live_component\n  module={MyAppWeb.Components.EditForm}\n  id=\"edit-form\"\n  {@edit_form}\n/>\n```\n\nThis pattern is especially useful for modal forms and other complex UI elements\nthat need to be controlled by a parent view. The parent can:\n\n- Initialize the component state: `assign(socket, :edit_form, %{item_id: \"123\", action: :edit})`\n- Reset the component: `assign(socket, :edit_form, nil)`\n- Update specific attributes: `update(socket, :edit_form, &Map.put(&1, :action, :new))`\n\nAll while maintaining proper type safety and state persistence according to the\ncomponent's defined attributes.","ref":"readme.html#component-properties-attr"},{"type":"extras","title":"Simplified State Updates (JSX.assign_state) - Livex","doc":"For components, Livex introduces JSX.assign_state (used within HEEx templates) as\na convenient way to update its own state properties. This is analogous to the\nsetter functions returned by useState in React functional components. It\nsimplifies common cases of updating component state directly from template\nevents without needing a full handle_event callback for simple assignments.\n\nExample:\n\n```elixir\ndefmodule MyAppWeb.Components.CounterButton do\n  use MyAppWeb, :livex_component\n\n  state :current_count, :integer\n  state :is_highlighted, :boolean\n\n  def pre_render(socket) do\n    # Set initial values using assign_new without dependencies\n    {:noreply,\n      socket\n      |> assign_new(:current_count, fn -> 0 end)\n      |> assign_new(:is_highlighted, fn -> false end)\n    }\n  end\n\n  def render(assigns) do\n    ~H\"\"\"\n     \n       \n        Count: {@current_count}\n       \n       \n        Increment\n       \n       \n        Toggle Highlight\n       \n       \n        Reset\n       \n     \n    \"\"\"\n  end\nend\n```\n\nClicking these buttons will directly update the :current_count and/or\n:is_highlighted state of the CounterButton component instance, triggering a\nre-render. JSX.assign_state can update one or multiple state fields. For more\ncomplex state transitions or side effects, you would still use handle_event.\n\nExample of conditional state updates:\n\n```elixir\n \n  {if @is_expanded, do: \"Close\", else: \"Expand\"}\n \n```","ref":"readme.html#simplified-state-updates-jsx-assign_state"},{"type":"extras","title":"Component Events (push_emit, phx-event) - Livex","doc":"Livex enhances event handling for components, allowing them to emit custom\nevents that parent LiveViews or LiveComponents can listen for, similar to how\nstandard phx-click and other events work.\n\n- **Emitting from the component's template**: Use JSX.emit(:event_name, value:\n  %{...}) within a phx-click or other event binding.\n- **Emitting from the component's Elixir code**: Use push_emit(socket,\n  :event_name, payload \\\\ %{}).\n\nExample:\n\nChild Component (MyAppWeb.Components.EditableField):\n\n```elixir\ndefmodule MyAppWeb.Components.EditableField do\n  use MyAppWeb, :livex_component\n\n  attr :field_id, :string, required: true\n  attr :initial_value, :string\n  state :current_value, :string\n  state :is_editing, :boolean\n\n  def pre_render(socket) do\n    # Initialize current_value from attr and set editing to false initially\n    {:noreply,\n      socket\n      |> assign_new(:current_value, [:initial_value], &(&1.initial_value))\n      |> assign_new(:is_editing, fn -> false end)\n    }\n  end\n\n  def render(assigns) do\n    ~H\"\"\"\n     \n      <%= if @is_editing do %>\n         \n         Save \n         Cancel \n      <% else %>\n         <%= @current_value %> \n         Edit \n      <% end %>\n     \n    \"\"\"\n  end\n\n  def handle_event(\"update_value\", %{\"value\" => new_val}, socket) do\n    {:noreply, assign(socket, :current_value, new_val)}\n  end\n\n  def handle_event(\"save_changes\", _, socket) do\n    # Emit an event to the parent with the field_id and new value\n    socket = push_emit(socket, :field_updated, %{id: socket.assigns.field_id, value: socket.assigns.current_value})\n    {:noreply, assign(socket, :is_editing, false)}\n  end\nend\n```\n\nParent View (MyAppWeb.SettingsView):\n\n```elixir\ndefmodule MyAppWeb.SettingsView do\n  use MyAppWeb, :livex_view\n\n  state :username, :string\n  state :email, :string\n\n  def pre_render(socket) do\n    # Set initial values for settings\n    {:noreply,\n      socket\n      |> assign_new(:username, fn -> \"User123\" end)\n      |> assign_new(:email, fn -> \"user@example.com\" end)\n    }\n  end\n\n  def render(assigns) do\n    ~H\"\"\"\n     \n       Settings \n       Username: \n      <.live_component\n        module={MyAppWeb.Components.EditableField}\n        id={\"username_field\"}\n        field_id=\"username\"\n        initial_value={@username}\n        phx-field_updated=\"handle_setting_change\"\n      />\n       Email: \n      <.live_component\n        module={MyAppWeb.Components.EditableField}\n        id={\"email_field\"}\n        field_id=\"email\"\n        initial_value={@email}\n        phx-field_updated=\"handle_setting_change\"\n      />\n     \n    \"\"\"\n  end\n\n  def handle_event(\"handle_setting_change\", %{\"id\" => \"username\", \"value\" => new_username}, socket) do\n    # Persist username change, update local state\n    {:noreply, assign(socket, :username, new_username)}\n  end\n\n  def handle_event(\"handle_setting_change\", %{\"id\" => \"email\", \"value\" => new_email}, socket) do\n    # Persist email change, update local state\n    {:noreply, assign(socket, :email, new_email)}\n  end\nend\n```","ref":"readme.html#component-events-push_emit-phx-event"},{"type":"extras","title":"Enable PubSub for Components (assign_topic) - Livex","doc":"** Not Implemented Yet! **\n\nLivex aims to make it easier for components to subscribe to PubSub topics\nthemselves, especially when the topic depends on the component's attrs or state.\n\nassign_topic/4 works similarly to assign_new/4, subscribing the component to a\nPubSub topic. The topic name can be dynamically generated based on dependencies.\nIf those dependencies change, the component would be re-subscribed to the new\ntopic.\n\n```elixir\ndefmodule MyAppWeb.Components.RealtimeDocumentStatus do\n  use MyAppWeb, :livex_component\n\n  attr :document_id, :uuid, required: true\n  state :status_message, :string\n\n  def pre_render(socket) do\n    {:noreply,\n     socket\n     # Set initial status\n     |> assign_new(:status_message, fn -> \"Connecting...\" end)\n     # Subscribe to topic based on attr\n     |> assign_topic(:doc_status_updates, [:document_id], fn assigns ->\n       # Dynamically generate topic based on document_id\n       \"document_updates:#{assigns.document_id}\"\n     end)}\n  end\n\n  # This will be called when a message is published to the subscribed topic\n  def handle_info({:doc_status_updates, %{message: msg}}, socket) do\n    # Update component state based on the PubSub message\n    {:noreply, assign(socket, :status_message, msg)}\n  end\n\n  def render(assigns) do\n    ~H\"\"\"\n     \n      Document <%= @document_id %>: <%= @status_message %>\n     \n    \"\"\"\n  end\nend\n```","ref":"readme.html#enable-pubsub-for-components-assign_topic"},{"type":"extras","title":"Summary - Livex","doc":"By combining these features, Livex aims to provide:\n\n- A cleaner conceptual model for LiveView and LiveComponent state and lifecycle.\n- Reduced boilerplate for common patterns like URL state management, derived\n  data, and component communication.\n- More declarative data flow, where changes to state or attrs automatically\n  propagate to derived data and streams via pre_render and dependency-aware\n  functions like assign_new and stream_new.\n- Avoidance of many manual push_patch calls, as state changes naturally lead to\n  re-renders.","ref":"readme.html#summary"},{"type":"extras","title":"Contributing - Livex","doc":"Contributions are welcome! This is an experimental project, so feel free to open\nissues or submit pull requests with ideas or improvements.","ref":"readme.html#contributing"},{"type":"extras","title":"License - Livex","doc":"This project is licensed under the MIT License - see the LICENSE file for\ndetails.","ref":"readme.html#license"},{"type":"extras","title":"Setup Instructions - Livex","doc":"1. Add Livex to your dependencies\n\nAdd livex to your dependencies in mix.exs:\n\n```elixir\ndef deps do\n  [\n    {:livex, \"~> 0.1.0\"}\n  ]\nend\n```\n\n2. Configure your web module\n\nUpdate your web module (typically lib/my_app_web.ex) to include Livex view and\ncomponent definitions. This usually involves defining livex_view/0 and\nlivex_component/0 functions that use Livex.LivexView or Livex.LivexComponent\nrespectively, along with Livex.JSX and any shared HTML helpers.\n\nExample lib/my_app_web.ex:\n\n```elixir\ndefmodule MyAppWeb do\n  # ... other functions like static_paths, router, channel, controller ...\n\n  def livex_view do\n    quote do\n      use Phoenix.LiveView,\n        layout: {MyAppWeb.Layouts, :app} # Or your desired layout\n\n      unquote(view_helpers()) # Keep your existing view_helpers\n\n      # Livex specific uses\n      use Livex.LivexView\n      use Livex.JSX # For JSX.emit, JSX.assign_state etc.\n    end\n  end\n\n  def livex_component do\n    quote do\n      use Phoenix.LiveComponent\n\n      # Livex specific uses\n      use Livex.LivexComponent\n      use Livex.JSX # For JSX.emit, JSX.assign_state etc.\n    end\n  end\n\n  def view_helpers do\n    quote do\n      use Phoenix.HTML\n      import Phoenix.LiveView.Helpers\n      import Phoenix.View\n      # ... other helpers like ErrorHelpers, Gettext, Routes, CoreComponents\n      # import MyAppWeb.CoreComponents # If you have them\n    end\n  end\n\n  # ... (rest of your web module, e.g., verified_routes)\nend\n```\n\n3. JavaScript Integration\n\nAdd these two lines to your assets/js/app.js file:\n\n```javascript\n// Add this import near your other imports (after LiveSocket import)\nimport { enhanceLiveSocket } from \"livex\";\n\n// Add this line after creating your LiveSocket instance but before connecting\n// Example:\n// let csrfToken = document.querySelector(\"meta[name='csrf-token']\").getAttribute(\"content\")\n// let liveSocket = new LiveSocket(\"/live\", Socket, {params: {_csrf_token: csrfToken}})\nliveSocket = enhanceLiveSocket(liveSocket); // Add this line\n// liveSocket.connect()\n```\n\n4. Using Livex in your application\n\nCreate LiveView modules using the new livex_view definition from your web module:\n\n```elixir\ndefmodule MyAppWeb.MyPageView do\n  use MyAppWeb, :livex_view # Assuming :livex_view is defined in MyAppWeb\n\n  state :item_id, :string, url?: true\n  state :is_editing_mode, :boolean\n\n  # Your view code...\nend\n```\n\nCreate LiveComponents using the new livex_component definition:\n\n```elixir\ndefmodule MyAppWeb.MyItemComponent do\n  use MyAppWeb, :livex_component # Assuming :livex_component is defined in MyAppWeb\n\n  attr :item_name, :string, required: true\n  state :show_details, :boolean\n\n  # Your component code...\nend\n```\n\n```\n\n```","ref":"readme.html#setup-instructions"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}